/*!****************************************************************************
@File           pvr2dutils.h

@Title          PVR2D tables utility header

@Author         Imagination Technologies

@Date           9/2/2010

@Copyright      Copyright 2001-2010 by Imagination Technologies Limited.
                All rights reserved. No part of this software, either material
                or conceptual may be copied or distributed, transmitted,
                transcribed, stored in a retrieval system or translated into
                any human or computer language in any form by any means,
                electronic, mechanical, manual or otherwise, or disclosed
                to third parties without the express written permission of
                Imagination Technologies Limited, Home Park Estate,
                Kings Langley, Hertfordshire, WD4 8LZ, U.K.

@Platform       Generic

@Description    PVR2D tables utility header

@DoxygenVer

******************************************************************************/

/******************************************************************************
Modifications :-
$Log: pvr2dutils.h $
******************************************************************************/


#ifdef INCLUDE_PVR2D_FORMAT_BITS_PER_PIXEL_TABLE
// Bits per pixel LUT for PVR2D formats
static const IMG_BYTE aPvr2DFormatBitsPerPixel[]=
{
				1,	//		PVR2D_1BPP			
				16,	//		PVR2D_RGB565		
				16,	//		PVR2D_ARGB4444		
				24,	//		PVR2D_RGB888		
				32,	//		PVR2D_ARGB8888		
				16,	//		PVR2D_ARGB1555		
				8,	//		PVR2D_ALPHA8		
				4,	//		PVR2D_ALPHA4		
				2,	//		PVR2D_PAL2			
				4,	//		PVR2D_PAL4			
				8,	//		PVR2D_PAL8			
				8,	//		PVR2D_U8			
				16,	//		PVR2D_U88			
				8,	//		PVR2D_S8			
				16,	//		PVR2D_YUV422_YUYV	
				16,	//		PVR2D_YUV422_UYVY	
				16,	//		PVR2D_YUV422_YVYU	
				16,	//		PVR2D_YUV422_VYUY	
				12,	//		PVR2D_YUV420_2PLANE	
				12,	//		PVR2D_YUV420_3PLANE	
				32,	//		PVR2D_2101010ARGB	
				24,	//		PVR2D_888RSGSBS		
				16,	//		PVR2D_16BPP_RAW		
				32,	//		PVR2D_32BPP_RAW		
				64,	//		PVR2D_64BPP_RAW		
				128 //		PVR2D_128BPP_RAW	
};
#define PVR2D_FORMAT_BITS_PER_PIXEL_MAX									PVR2D_NO_OF_FORMATS
#endif // #ifdef INCLUDE_PVR2D_FORMAT_BITS_PER_PIXEL_TABLE



#ifdef INCLUDE_PVRSRV_FORMAT_BITS_PER_PIXEL_TABLE
// Bits per pixel for PVRSRV formats
static const IMG_BYTE aPvrsrvFormatBitsPerPixel[]=
{
				0,	//		PVRSRV_PIXEL_FORMAT_UNKNOWN				=  0,
				16, //		PVRSRV_PIXEL_FORMAT_RGB565				=  1,
				16, //		PVRSRV_PIXEL_FORMAT_RGB555				=  2,
				24, //		PVRSRV_PIXEL_FORMAT_RGB888				=  3,
				24, //		PVRSRV_PIXEL_FORMAT_BGR888				=  4,
				0,	//
				0,	//
				0,	//
				8,	//		PVRSRV_PIXEL_FORMAT_GREY_SCALE			=  8,
				0,	//
				0,	//
				0,	//
				0,	//
				12, //		PVRSRV_PIXEL_FORMAT_PAL12				= 13,
				8,	//		PVRSRV_PIXEL_FORMAT_PAL8				= 14,
				4,	//		PVRSRV_PIXEL_FORMAT_PAL4				= 15,
				2,	//		PVRSRV_PIXEL_FORMAT_PAL2				= 16,
				1,	//		PVRSRV_PIXEL_FORMAT_PAL1				= 17,
				16, //		PVRSRV_PIXEL_FORMAT_ARGB1555			= 18,
				16, //		PVRSRV_PIXEL_FORMAT_ARGB4444			= 19,
				32, //		PVRSRV_PIXEL_FORMAT_ARGB8888			= 20,
				32, //		PVRSRV_PIXEL_FORMAT_ABGR8888			= 21,
				0,	//		PVRSRV_PIXEL_FORMAT_YV12				= 22,
				0,	//		PVRSRV_PIXEL_FORMAT_I420				= 23,
				0,	//
				0,	//		PVRSRV_PIXEL_FORMAT_IMC2				= 25,
				32, //		PVRSRV_PIXEL_FORMAT_XRGB8888			= 26,
				32, //		PVRSRV_PIXEL_FORMAT_XBGR8888			= 27,
				32, //		PVRSRV_PIXEL_FORMAT_BGRA8888			= 28,
				16, //		PVRSRV_PIXEL_FORMAT_XRGB4444			= 29,
				16, //		PVRSRV_PIXEL_FORMAT_ARGB8332			= 30,
				32, //		PVRSRV_PIXEL_FORMAT_A2RGB10				= 31,
				32, //		PVRSRV_PIXEL_FORMAT_A2BGR10				= 32,
				8,	//		PVRSRV_PIXEL_FORMAT_P8					= 33,
				8,	//		PVRSRV_PIXEL_FORMAT_L8					= 34,
				16, //		PVRSRV_PIXEL_FORMAT_A8L8				= 35,
				8,	//		PVRSRV_PIXEL_FORMAT_A4L4				= 36,
				16, //		PVRSRV_PIXEL_FORMAT_L16					= 37,
				16, //		PVRSRV_PIXEL_FORMAT_L6V5U5				= 38,
				16, //		PVRSRV_PIXEL_FORMAT_V8U8				= 39,
				32, //		PVRSRV_PIXEL_FORMAT_V16U16				= 40,
				32, //		PVRSRV_PIXEL_FORMAT_QWVU8888			= 41,
				32, //		PVRSRV_PIXEL_FORMAT_XLVU8888			= 42,
				64, //		PVRSRV_PIXEL_FORMAT_QWVU16				= 43,
				16, //		PVRSRV_PIXEL_FORMAT_D16					= 44,
				32, //		PVRSRV_PIXEL_FORMAT_D24S8				= 45,
				32, //		PVRSRV_PIXEL_FORMAT_D24X8				= 46,
				64, //		PVRSRV_PIXEL_FORMAT_ABGR16				= 47,
				64, //		PVRSRV_PIXEL_FORMAT_ABGR16F				= 48,
				0,	//		PVRSRV_PIXEL_FORMAT_ABGR32				= 49,
				128, //		PVRSRV_PIXEL_FORMAT_ABGR32F				= 50,
				0,	//		PVRSRV_PIXEL_FORMAT_B10GR11				= 51,
				16, //		PVRSRV_PIXEL_FORMAT_GR88				= 52,
				0,	//		PVRSRV_PIXEL_FORMAT_BGR32				= 53,
				0,	//		PVRSRV_PIXEL_FORMAT_GR32				= 54,
				0,	//		PVRSRV_PIXEL_FORMAT_E5BGR9				= 55,
				4,	//		PVRSRV_PIXEL_FORMAT_RESERVED1			= 56,
				8,	//		PVRSRV_PIXEL_FORMAT_RESERVED2			= 57,
				8,	//		PVRSRV_PIXEL_FORMAT_RESERVED3			= 58,
				8,	//		PVRSRV_PIXEL_FORMAT_RESERVED4			= 59,
				8,	//		PVRSRV_PIXEL_FORMAT_RESERVED5			= 60,
				16, //		PVRSRV_PIXEL_FORMAT_R8G8_B8G8			= 61,
				16, //		PVRSRV_PIXEL_FORMAT_G8R8_G8B8			= 62,
				0,	//		PVRSRV_PIXEL_FORMAT_NV11				= 63,
				12, //		PVRSRV_PIXEL_FORMAT_NV12				= 64,
				16, //		PVRSRV_PIXEL_FORMAT_YUY2				= 65,
				0,	//		PVRSRV_PIXEL_FORMAT_YUV420				= 66,
				0,	//		PVRSRV_PIXEL_FORMAT_YUV444				= 67,
				0,	//		PVRSRV_PIXEL_FORMAT_VUY444				= 68,
				16, //		PVRSRV_PIXEL_FORMAT_YUYV				= 69,
				16, //		PVRSRV_PIXEL_FORMAT_YVYU				= 70,
				16, //		PVRSRV_PIXEL_FORMAT_UYVY				= 71,
				16, //		PVRSRV_PIXEL_FORMAT_VYUY				= 72,
				16, //		PVRSRV_PIXEL_FORMAT_FOURCC_ORG_UYVY		= 73,	
				16, //		PVRSRV_PIXEL_FORMAT_FOURCC_ORG_YUYV		= 74,	
				16, //		PVRSRV_PIXEL_FORMAT_FOURCC_ORG_YVYU		= 75,	
				16, //		PVRSRV_PIXEL_FORMAT_FOURCC_ORG_VYUY		= 76,	
				32, //		PVRSRV_PIXEL_FORMAT_FOURCC_ORG_AYUV		= 77,	
				128, //		PVRSRV_PIXEL_FORMAT_A32B32G32R32		= 78,	
				128, //		PVRSRV_PIXEL_FORMAT_A32B32G32R32F		= 79,	
				128, //		PVRSRV_PIXEL_FORMAT_A32B32G32R32_UINT	= 80,	
				128, //		PVRSRV_PIXEL_FORMAT_A32B32G32R32_SINT	= 81,	
				96, //		PVRSRV_PIXEL_FORMAT_B32G32R32			= 82,	
				96, //		PVRSRV_PIXEL_FORMAT_B32G32R32F			= 83,	
				96, //		PVRSRV_PIXEL_FORMAT_B32G32R32_UINT		= 84,	
				96, //		PVRSRV_PIXEL_FORMAT_B32G32R32_SINT		= 85,	
				64, //		PVRSRV_PIXEL_FORMAT_G32R32				= 86,	
				64, //		PVRSRV_PIXEL_FORMAT_G32R32F				= 87,	
				64, //		PVRSRV_PIXEL_FORMAT_G32R32_UINT			= 88,	
				64, //		PVRSRV_PIXEL_FORMAT_G32R32_SINT			= 89,	
				32, //		PVRSRV_PIXEL_FORMAT_D32F				= 90,	
				32, //		PVRSRV_PIXEL_FORMAT_R32					= 91,	
				32, //		PVRSRV_PIXEL_FORMAT_R32F				= 92,	
				32, //		PVRSRV_PIXEL_FORMAT_R32_UINT			= 93,	
				32, //		PVRSRV_PIXEL_FORMAT_R32_SINT			= 94,	
				64, //		PVRSRV_PIXEL_FORMAT_A16B16G16R16		= 95,	
				64, //		PVRSRV_PIXEL_FORMAT_A16B16G16R16F		= 96,	
				64, //		PVRSRV_PIXEL_FORMAT_A16B16G16R16_SINT	= 97,	
				64, //		PVRSRV_PIXEL_FORMAT_A16B16G16R16_SNORM	= 98,	
				64, //		PVRSRV_PIXEL_FORMAT_A16B16G16R16_UINT	= 99,	
				64, //		PVRSRV_PIXEL_FORMAT_A16B16G16R16_UNORM	= 100,	
				32, //		PVRSRV_PIXEL_FORMAT_G16R16				= 101,	
				32, //		PVRSRV_PIXEL_FORMAT_G16R16F				= 102,	
				32, //		PVRSRV_PIXEL_FORMAT_G16R16_UINT			= 103,	
				32, //		PVRSRV_PIXEL_FORMAT_G16R16_UNORM		= 104,	
				32, //		PVRSRV_PIXEL_FORMAT_G16R16_SINT			= 105,	
				32, //		PVRSRV_PIXEL_FORMAT_G16R16_SNORM		= 106,	
				16, //		PVRSRV_PIXEL_FORMAT_R16					= 107,	
				16, //		PVRSRV_PIXEL_FORMAT_R16F				= 108,	
				16, //		PVRSRV_PIXEL_FORMAT_R16_UINT			= 109,	
				16, //		PVRSRV_PIXEL_FORMAT_R16_UNORM			= 110,	
				16, //		PVRSRV_PIXEL_FORMAT_R16_SINT			= 111,	
				16, //		PVRSRV_PIXEL_FORMAT_R16_SNORM			= 112,	
				32, //		PVRSRV_PIXEL_FORMAT_X8R8G8B8			= 113,	
				32, //		PVRSRV_PIXEL_FORMAT_X8R8G8B8_UNORM		= 114,	
				32, //		PVRSRV_PIXEL_FORMAT_X8R8G8B8_UNORM_SRGB	= 115,	
				32, //		PVRSRV_PIXEL_FORMAT_A8R8G8B8			= 116,	
				32, //		PVRSRV_PIXEL_FORMAT_A8R8G8B8_UNORM		= 117,	
				32, //		PVRSRV_PIXEL_FORMAT_A8R8G8B8_UNORM_SRGB	= 118,	
				32, //		PVRSRV_PIXEL_FORMAT_A8B8G8R8			= 119,	
				32, //		PVRSRV_PIXEL_FORMAT_A8B8G8R8_UINT		= 120,	
				32, //		PVRSRV_PIXEL_FORMAT_A8B8G8R8_UNORM		= 121,	
				32, //		PVRSRV_PIXEL_FORMAT_A8B8G8R8_UNORM_SRGB	= 122,	
				32, //		PVRSRV_PIXEL_FORMAT_A8B8G8R8_SINT		= 123,	
				32, //		PVRSRV_PIXEL_FORMAT_A8B8G8R8_SNORM		= 124,	
				16, //		PVRSRV_PIXEL_FORMAT_G8R8				= 125,	
				16, //		PVRSRV_PIXEL_FORMAT_G8R8_UINT			= 126,	
				16, //		PVRSRV_PIXEL_FORMAT_G8R8_UNORM			= 127,	
				16, //		PVRSRV_PIXEL_FORMAT_G8R8_SINT			= 128,	
				16, //		PVRSRV_PIXEL_FORMAT_G8R8_SNORM			= 129,	
				8,	//		PVRSRV_PIXEL_FORMAT_A8					= 130,	
				8,	//		PVRSRV_PIXEL_FORMAT_R8					= 131,	
				8,	//		PVRSRV_PIXEL_FORMAT_R8_UINT				= 132,	
				8,	//		PVRSRV_PIXEL_FORMAT_R8_UNORM			= 133,	
				8,	//		PVRSRV_PIXEL_FORMAT_R8_SINT				= 134,	
				8,	//		PVRSRV_PIXEL_FORMAT_R8_SNORM			= 135,	
				32, //		PVRSRV_PIXEL_FORMAT_A2B10G10R10			= 136,	
				32, //		PVRSRV_PIXEL_FORMAT_A2B10G10R10_UNORM	= 137,	
				32, //		PVRSRV_PIXEL_FORMAT_A2B10G10R10_UINT	= 138,	
				32, //		PVRSRV_PIXEL_FORMAT_B10G11R11			= 139,	
				32, //		PVRSRV_PIXEL_FORMAT_B10G11R11F			= 140,	
				0,	//		PVRSRV_PIXEL_FORMAT_X24G8R32			= 141,	
				32, //		PVRSRV_PIXEL_FORMAT_G8R24				= 142,	
				32, //		PVRSRV_PIXEL_FORMAT_X8R24				= 143,
				32, //		PVRSRV_PIXEL_FORMAT_E5B9G9R9			= 144,	
				0,	//		PVRSRV_PIXEL_FORMAT_R1					= 145,	
				4,	//		PVRSRV_PIXEL_FORMAT_RESERVED6			= 146,	
				4,	//		PVRSRV_PIXEL_FORMAT_RESERVED7			= 147,	
				4,	//		PVRSRV_PIXEL_FORMAT_RESERVED8			= 148,	
				8,	//		PVRSRV_PIXEL_FORMAT_RESERVED9			= 149,	
				8,	//		PVRSRV_PIXEL_FORMAT_RESERVED10			= 150,	
				8,	//		PVRSRV_PIXEL_FORMAT_RESERVED11			= 151,	
				8,	//		PVRSRV_PIXEL_FORMAT_RESERVED12			= 152,	
				8,	//		PVRSRV_PIXEL_FORMAT_RESERVED13			= 153,	
				8,	//		PVRSRV_PIXEL_FORMAT_RESERVED14			= 154,	
				4,	//		PVRSRV_PIXEL_FORMAT_RESERVED15			= 155,	
				4,	//		PVRSRV_PIXEL_FORMAT_RESERVED16			= 156,	
				4,	//		PVRSRV_PIXEL_FORMAT_RESERVED17			= 157,	
				8,	//		PVRSRV_PIXEL_FORMAT_RESERVED18			= 158,	
				8,	//		PVRSRV_PIXEL_FORMAT_RESERVED19			= 159,
				8,	//		PVRSRV_PIXEL_FORMAT_RESERVED20			= 160,
				0,	//		PVRSRV_PIXEL_FORMAT_UBYTE4				= 161,	
				0,	//		PVRSRV_PIXEL_FORMAT_SHORT4				= 162,	
				0,	//		PVRSRV_PIXEL_FORMAT_SHORT4N				= 163,	
				0,	//		PVRSRV_PIXEL_FORMAT_USHORT4N			= 164,	
				0,	//		PVRSRV_PIXEL_FORMAT_SHORT2N				= 165,	
				0,	//		PVRSRV_PIXEL_FORMAT_SHORT2				= 166,	
				0,	//		PVRSRV_PIXEL_FORMAT_USHORT2N			= 167,	
				0,	//		PVRSRV_PIXEL_FORMAT_UDEC3				= 168,	
				0,	//		PVRSRV_PIXEL_FORMAT_DEC3N				= 169,	
				0,	//		PVRSRV_PIXEL_FORMAT_F16_2				= 170,	
				0,	//		PVRSRV_PIXEL_FORMAT_F16_4				= 171,	
				0,	//		PVRSRV_PIXEL_FORMAT_L_F16				= 172,
				0,	//		PVRSRV_PIXEL_FORMAT_L_F16_REP			= 173,
				0,	//		PVRSRV_PIXEL_FORMAT_L_F16_A_F16			= 174,
				0,	//		PVRSRV_PIXEL_FORMAT_A_F16				= 175,
				0,	//		PVRSRV_PIXEL_FORMAT_B16G16R16F			= 176,
				0,	//		PVRSRV_PIXEL_FORMAT_L_F32				= 177,
				0,	//		PVRSRV_PIXEL_FORMAT_A_F32				= 178,
				0,	//		PVRSRV_PIXEL_FORMAT_L_F32_A_F32			= 179,
				2,	//		PVRSRV_PIXEL_FORMAT_PVRTC2				= 180,
				4,	//		PVRSRV_PIXEL_FORMAT_PVRTC4				= 181,
				8,	//		PVRSRV_PIXEL_FORMAT_PVRTCII2			= 182,
				4,	//		PVRSRV_PIXEL_FORMAT_PVRTCII4			= 183,
				4,	//		PVRSRV_PIXEL_FORMAT_PVRTCIII			= 184,
				0,	//		PVRSRV_PIXEL_FORMAT_PVRO8				= 185,
				0,	//		PVRSRV_PIXEL_FORMAT_PVRO88				= 186,
				32, //		PVRSRV_PIXEL_FORMAT_PT1					= 187,
				64, //		PVRSRV_PIXEL_FORMAT_PT2					= 188,
				128, //		PVRSRV_PIXEL_FORMAT_PT4					= 189,
				0,	//		PVRSRV_PIXEL_FORMAT_PT8					= 190,
				16, //		PVRSRV_PIXEL_FORMAT_PTW					= 191,
				8,	//		PVRSRV_PIXEL_FORMAT_PTB					= 192,
				8,	//		PVRSRV_PIXEL_FORMAT_MONO8				= 193,
				16, //		PVRSRV_PIXEL_FORMAT_MONO16				= 194,
				16, //		PVRSRV_PIXEL_FORMAT_C0_YUYV				= 195,
				16, //		PVRSRV_PIXEL_FORMAT_C0_UYVY				= 196,
				16, //		PVRSRV_PIXEL_FORMAT_C0_YVYU				= 197,
				16, //		PVRSRV_PIXEL_FORMAT_C0_VYUY				= 198,
				16, //		PVRSRV_PIXEL_FORMAT_C1_YUYV				= 199,
				16, //		PVRSRV_PIXEL_FORMAT_C1_UYVY				= 200,
				16, //		PVRSRV_PIXEL_FORMAT_C1_YVYU				= 201,
				16, //		PVRSRV_PIXEL_FORMAT_C1_VYUY				= 202,
				12, //		PVRSRV_PIXEL_FORMAT_C0_YUV420_2P_UV		= 203,
				12, //		PVRSRV_PIXEL_FORMAT_C0_YUV420_2P_VU		= 204,
				12, //		PVRSRV_PIXEL_FORMAT_C0_YUV420_3P		= 205,
				12, //		PVRSRV_PIXEL_FORMAT_C1_YUV420_2P_UV		= 206,
				12, //		PVRSRV_PIXEL_FORMAT_C1_YUV420_2P_VU		= 207,
				12, //		PVRSRV_PIXEL_FORMAT_C1_YUV420_3P		= 208,
				32, //		PVRSRV_PIXEL_FORMAT_A2B10G10R10F		= 209,
				24, //		PVRSRV_PIXEL_FORMAT_B8G8R8_SINT			= 210,
				32, //		PVRSRV_PIXEL_FORMAT_PVRF32SIGNMASK		= 211,
				16, //		PVRSRV_PIXEL_FORMAT_ABGR4444			= 212,	
				16, //		PVRSRV_PIXEL_FORMAT_ABGR1555			= 213,
				16, //		PVRSRV_PIXEL_FORMAT_BGR565				= 214,			
};
#define PVRSRV_PIXEL_FORMAT_MAX										214
#endif // #ifdef INCLUDE_PVRSRV_FORMAT_BITS_PER_PIXEL_TABLE


#if defined(INCLUDE_PVRSRV_PDUMP_FORMATS) && defined (PDUMP)
// Mapping of PVRSRV to PDUMP formats
static const IMG_UINT32 aPvrsrvPdumpFormats[]=
{
				0,										//		PVRSRV_PIXEL_FORMAT_UNKNOWN				=  0,
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB565,		//		PVRSRV_PIXEL_FORMAT_RGB565				=  1,
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB555,		//		PVRSRV_PIXEL_FORMAT_RGB555				=  2,
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB888,		//		PVRSRV_PIXEL_FORMAT_RGB888				=  3,
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB888,		//		PVRSRV_PIXEL_FORMAT_BGR888				=  4,
				0,
				0,
				0,
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB8,			//		PVRSRV_PIXEL_FORMAT_GREY_SCALE			=  8,
				0,
				0,
				0,
				0,
				0,										//		PVRSRV_PIXEL_FORMAT_PAL12				= 13,
				0,										//		PVRSRV_PIXEL_FORMAT_PAL8				= 14,
				0,										//		PVRSRV_PIXEL_FORMAT_PAL4				= 15,
				0,										//		PVRSRV_PIXEL_FORMAT_PAL2				= 16,
				0,										//		PVRSRV_PIXEL_FORMAT_PAL1				= 17,
				PVRSRV_PDUMP_PIXEL_FORMAT_ARGB1555,		//		PVRSRV_PIXEL_FORMAT_ARGB1555			= 18,
				PVRSRV_PDUMP_PIXEL_FORMAT_ARGB4444,		//		PVRSRV_PIXEL_FORMAT_ARGB4444			= 19,
				PVRSRV_PDUMP_PIXEL_FORMAT_ARGB8888,		//		PVRSRV_PIXEL_FORMAT_ARGB8888			= 20,
				PVRSRV_PDUMP_PIXEL_FORMAT_ABGR8888,		//		PVRSRV_PIXEL_FORMAT_ABGR8888			= 21,
				0,										//		PVRSRV_PIXEL_FORMAT_YV12				= 22,
				0,										//		PVRSRV_PIXEL_FORMAT_I420				= 23,
				0,										//
				0,										//		PVRSRV_PIXEL_FORMAT_IMC2				= 25,
				PVRSRV_PDUMP_PIXEL_FORMAT_ARGB8888,		//		PVRSRV_PIXEL_FORMAT_XRGB8888			= 26,
				PVRSRV_PDUMP_PIXEL_FORMAT_ARGB8888,		//		PVRSRV_PIXEL_FORMAT_XBGR8888			= 27,
				PVRSRV_PDUMP_PIXEL_FORMAT_BGRA8888,		//		PVRSRV_PIXEL_FORMAT_BGRA8888			= 28,
				PVRSRV_PDUMP_PIXEL_FORMAT_ARGB4444,		//		PVRSRV_PIXEL_FORMAT_XRGB4444			= 29,
				PVRSRV_PDUMP_PIXEL_FORMAT_ARGB8332,		//		PVRSRV_PIXEL_FORMAT_ARGB8332			= 30,
				0,										//		PVRSRV_PIXEL_FORMAT_A2RGB10				= 31,
				0,										//		PVRSRV_PIXEL_FORMAT_A2BGR10				= 32,
				0,										//		PVRSRV_PIXEL_FORMAT_P8					= 33,
				0,										//		PVRSRV_PIXEL_FORMAT_L8					= 34,
				0,										//		PVRSRV_PIXEL_FORMAT_A8L8				= 35,
				0,										//		PVRSRV_PIXEL_FORMAT_A4L4				= 36,
				0,										//		PVRSRV_PIXEL_FORMAT_L16					= 37,
				0,										//		PVRSRV_PIXEL_FORMAT_L6V5U5				= 38,
				0,										//		PVRSRV_PIXEL_FORMAT_V8U8				= 39,
				0,										//		PVRSRV_PIXEL_FORMAT_V16U16				= 40,
				0,										//		PVRSRV_PIXEL_FORMAT_QWVU8888			= 41,
				0,										//		PVRSRV_PIXEL_FORMAT_XLVU8888			= 42,
				0,										//		PVRSRV_PIXEL_FORMAT_QWVU16				= 43,
				0,										//		PVRSRV_PIXEL_FORMAT_D16					= 44,
				0,										//		PVRSRV_PIXEL_FORMAT_D24S8				= 45,
				0,										//		PVRSRV_PIXEL_FORMAT_D24X8				= 46,
				0,										//		PVRSRV_PIXEL_FORMAT_ABGR16				= 47,
				0,										//		PVRSRV_PIXEL_FORMAT_ABGR16F				= 48,
				0,										//		PVRSRV_PIXEL_FORMAT_ABGR32				= 49,
				0,										//		PVRSRV_PIXEL_FORMAT_ABGR32F				= 50,
				0,										//		PVRSRV_PIXEL_FORMAT_B10GR11				= 51,
				0,										//		PVRSRV_PIXEL_FORMAT_GR88				= 52,
				0,										//		PVRSRV_PIXEL_FORMAT_BGR32				= 53,
				0,										//		PVRSRV_PIXEL_FORMAT_GR32				= 54,
				0,										//		PVRSRV_PIXEL_FORMAT_E5BGR9				= 55,
				0,										//		PVRSRV_PIXEL_FORMAT_RESERVED1			= 56,
				0,										//		PVRSRV_PIXEL_FORMAT_RESERVED2			= 57,
				0,										//		PVRSRV_PIXEL_FORMAT_RESERVED3			= 58,
				0,										//		PVRSRV_PIXEL_FORMAT_RESERVED4			= 59,
				0,										//		PVRSRV_PIXEL_FORMAT_RESERVED5			= 60,
				0,										//		PVRSRV_PIXEL_FORMAT_R8G8_B8G8			= 61,
				0,										//		PVRSRV_PIXEL_FORMAT_G8R8_G8B8			= 62,
				0,										//		PVRSRV_PIXEL_FORMAT_NV11				= 63,
				0,										//		PVRSRV_PIXEL_FORMAT_NV12				= 64,
				0,										//		PVRSRV_PIXEL_FORMAT_YUY2				= 65,
				0,										//		PVRSRV_PIXEL_FORMAT_YUV420				= 66,
				0,										//		PVRSRV_PIXEL_FORMAT_YUV444				= 67,
				0,										//		PVRSRV_PIXEL_FORMAT_VUY444				= 68,
				PVRSRV_PDUMP_PIXEL_FORMAT_Y0UY1V_8888,	//		PVRSRV_PIXEL_FORMAT_YUYV				= 69,
				PVRSRV_PDUMP_PIXEL_FORMAT_Y0VY1U_8888,	//		PVRSRV_PIXEL_FORMAT_YVYU				= 70,
				PVRSRV_PDUMP_PIXEL_FORMAT_UY0VY1_8888,	//		PVRSRV_PIXEL_FORMAT_UYVY				= 71,
				PVRSRV_PDUMP_PIXEL_FORMAT_VY0UY1_8888,	//		PVRSRV_PIXEL_FORMAT_VYUY				= 72,
				PVRSRV_PDUMP_PIXEL_FORMAT_UY0VY1_8888,	//		PVRSRV_PIXEL_FORMAT_FOURCC_ORG_UYVY		= 73,	
				PVRSRV_PDUMP_PIXEL_FORMAT_Y0UY1V_8888,	//		PVRSRV_PIXEL_FORMAT_FOURCC_ORG_YUYV		= 74,	
				PVRSRV_PDUMP_PIXEL_FORMAT_Y0VY1U_8888,	//		PVRSRV_PIXEL_FORMAT_FOURCC_ORG_YVYU		= 75,	
				PVRSRV_PDUMP_PIXEL_FORMAT_VY0UY1_8888,	//		PVRSRV_PIXEL_FORMAT_FOURCC_ORG_VYUY		= 76,	
				PVRSRV_PDUMP_PIXEL_FORMAT_AYUV8888,		//		PVRSRV_PIXEL_FORMAT_FOURCC_ORG_AYUV		= 77,	
				0,										//		PVRSRV_PIXEL_FORMAT_A32B32G32R32		= 78,	
				0,										//		PVRSRV_PIXEL_FORMAT_A32B32G32R32F		= 79,	
				0,										//		PVRSRV_PIXEL_FORMAT_A32B32G32R32_UINT	= 80,	
				0,										//		PVRSRV_PIXEL_FORMAT_A32B32G32R32_SINT	= 81,	
				0,										//		PVRSRV_PIXEL_FORMAT_B32G32R32			= 82,	
				0,										//		PVRSRV_PIXEL_FORMAT_B32G32R32F			= 83,	
				0,										//		PVRSRV_PIXEL_FORMAT_B32G32R32_UINT		= 84,	
				0,										//		PVRSRV_PIXEL_FORMAT_B32G32R32_SINT		= 85,	
				0,										//		PVRSRV_PIXEL_FORMAT_G32R32				= 86,	
				0,										//		PVRSRV_PIXEL_FORMAT_G32R32F				= 87,	
				0,										//		PVRSRV_PIXEL_FORMAT_G32R32_UINT			= 88,	
				0,										//		PVRSRV_PIXEL_FORMAT_G32R32_SINT			= 89,	
				0, 										//		PVRSRV_PIXEL_FORMAT_D32F				= 90,	
				0, 										//		PVRSRV_PIXEL_FORMAT_R32					= 91,	
				0, 										//		PVRSRV_PIXEL_FORMAT_R32F				= 92,	
				0, 										//		PVRSRV_PIXEL_FORMAT_R32_UINT			= 93,	
				0, 										//		PVRSRV_PIXEL_FORMAT_R32_SINT			= 94,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A16B16G16R16		= 95,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A16B16G16R16F		= 96,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A16B16G16R16_SINT	= 97,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A16B16G16R16_SNORM	= 98,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A16B16G16R16_UINT	= 99,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A16B16G16R16_UNORM	= 100,	
				0, 										//		PVRSRV_PIXEL_FORMAT_G16R16				= 101,	
				0, 										//		PVRSRV_PIXEL_FORMAT_G16R16F				= 102,	
				0, 										//		PVRSRV_PIXEL_FORMAT_G16R16_UINT			= 103,	
				0, 										//		PVRSRV_PIXEL_FORMAT_G16R16_UNORM		= 104,	
				0, 										//		PVRSRV_PIXEL_FORMAT_G16R16_SINT			= 105,	
				0, 										//		PVRSRV_PIXEL_FORMAT_G16R16_SNORM		= 106,	
				0, 										//		PVRSRV_PIXEL_FORMAT_R16					= 107,	
				0, 										//		PVRSRV_PIXEL_FORMAT_R16F				= 108,	
				0, 										//		PVRSRV_PIXEL_FORMAT_R16_UINT			= 109,	
				0, 										//		PVRSRV_PIXEL_FORMAT_R16_UNORM			= 110,	
				0, 										//		PVRSRV_PIXEL_FORMAT_R16_SINT			= 111,	
				0, 										//		PVRSRV_PIXEL_FORMAT_R16_SNORM			= 112,	
				0, 										//		PVRSRV_PIXEL_FORMAT_X8R8G8B8			= 113,	
				0, 										//		PVRSRV_PIXEL_FORMAT_X8R8G8B8_UNORM		= 114,	
				0, 										//		PVRSRV_PIXEL_FORMAT_X8R8G8B8_UNORM_SRGB	= 115,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A8R8G8B8			= 116,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A8R8G8B8_UNORM		= 117,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A8R8G8B8_UNORM_SRGB	= 118,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A8B8G8R8			= 119,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A8B8G8R8_UINT		= 120,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A8B8G8R8_UNORM		= 121,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A8B8G8R8_UNORM_SRGB	= 122,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A8B8G8R8_SINT		= 123,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A8B8G8R8_SNORM		= 124,	
				0, 										//		PVRSRV_PIXEL_FORMAT_G8R8				= 125,	
				0, 										//		PVRSRV_PIXEL_FORMAT_G8R8_UINT			= 126,	
				0, 										//		PVRSRV_PIXEL_FORMAT_G8R8_UNORM			= 127,	
				0, 										//		PVRSRV_PIXEL_FORMAT_G8R8_SINT			= 128,	
				0, 										//		PVRSRV_PIXEL_FORMAT_G8R8_SNORM			= 129,	
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB8,			//		PVRSRV_PIXEL_FORMAT_A8					= 130,	
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB8,			//		PVRSRV_PIXEL_FORMAT_R8					= 131,	
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB8,			//		PVRSRV_PIXEL_FORMAT_R8_UINT				= 132,	
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB8,			//		PVRSRV_PIXEL_FORMAT_R8_UNORM			= 133,	
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB8,			//		PVRSRV_PIXEL_FORMAT_R8_SINT				= 134,	
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB8,			//		PVRSRV_PIXEL_FORMAT_R8_SNORM			= 135,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A2B10G10R10			= 136,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A2B10G10R10_UNORM	= 137,	
				0, 										//		PVRSRV_PIXEL_FORMAT_A2B10G10R10_UINT	= 138,	
				0, 										//		PVRSRV_PIXEL_FORMAT_B10G11R11			= 139,	
				0, 										//		PVRSRV_PIXEL_FORMAT_B10G11R11F			= 140,	
				0, 										//		PVRSRV_PIXEL_FORMAT_X24G8R32			= 141,	
				0, 										//		PVRSRV_PIXEL_FORMAT_G8R24				= 142,	
				0, 										//		PVRSRV_PIXEL_FORMAT_X8R24				= 143,
				0, 										//		PVRSRV_PIXEL_FORMAT_E5B9G9R9			= 144,	
				0, 										//		PVRSRV_PIXEL_FORMAT_R1					= 145,	
				0, 										//		PVRSRV_PIXEL_FORMAT_RESERVED6			= 146,	
				0, 										//		PVRSRV_PIXEL_FORMAT_RESERVED7			= 147,	
				0, 										//		PVRSRV_PIXEL_FORMAT_RESERVED8			= 148,	
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB8,			//		PVRSRV_PIXEL_FORMAT_RESERVED9			= 149,	
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB8,			//		PVRSRV_PIXEL_FORMAT_RESERVED10			= 150,	
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB8,			//		PVRSRV_PIXEL_FORMAT_RESERVED11			= 151,	
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB8,			//		PVRSRV_PIXEL_FORMAT_RESERVED12			= 152,	
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB8,			//		PVRSRV_PIXEL_FORMAT_RESERVED13			= 153,	
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB8,			//		PVRSRV_PIXEL_FORMAT_RESERVED14			= 154,	
				0, 										//		PVRSRV_PIXEL_FORMAT_RESERVED15			= 155,	
				0, 										//		PVRSRV_PIXEL_FORMAT_RESERVED16			= 156,	
				0, 										//		PVRSRV_PIXEL_FORMAT_RESERVED17			= 157,	
				0, 										//		PVRSRV_PIXEL_FORMAT_RESERVED18			= 158,	
				0, 										//		PVRSRV_PIXEL_FORMAT_RESERVED19			= 159,
				0, 										//		PVRSRV_PIXEL_FORMAT_RESERVED20			= 160,
				0, 										//		PVRSRV_PIXEL_FORMAT_UBYTE4				= 161,	
				0, 										//		PVRSRV_PIXEL_FORMAT_SHORT4				= 162,	
				0, 										//		PVRSRV_PIXEL_FORMAT_SHORT4N				= 163,	
				0, 										//		PVRSRV_PIXEL_FORMAT_USHORT4N			= 164,	
				0, 										//		PVRSRV_PIXEL_FORMAT_SHORT2N				= 165,	
				0, 										//		PVRSRV_PIXEL_FORMAT_SHORT2				= 166,	
				0, 										//		PVRSRV_PIXEL_FORMAT_USHORT2N			= 167,	
				0, 										//		PVRSRV_PIXEL_FORMAT_UDEC3				= 168,	
				0, 										//		PVRSRV_PIXEL_FORMAT_DEC3N				= 169,	
				0, 										//		PVRSRV_PIXEL_FORMAT_F16_2				= 170,	
				0, 										//		PVRSRV_PIXEL_FORMAT_F16_4				= 171,	
				0, 										//		PVRSRV_PIXEL_FORMAT_L_F16				= 172,
				0, 										//		PVRSRV_PIXEL_FORMAT_L_F16_REP			= 173,
				0, 										//		PVRSRV_PIXEL_FORMAT_L_F16_A_F16			= 174,
				0, 										//		PVRSRV_PIXEL_FORMAT_A_F16				= 175,
				0, 										//		PVRSRV_PIXEL_FORMAT_B16G16R16F			= 176,
				0, 										//		PVRSRV_PIXEL_FORMAT_L_F32				= 177,
				0, 										//		PVRSRV_PIXEL_FORMAT_A_F32				= 178,
				0, 										//		PVRSRV_PIXEL_FORMAT_L_F32_A_F32			= 179,
				0, 										//		PVRSRV_PIXEL_FORMAT_PVRTC2				= 180,
				0, 										//		PVRSRV_PIXEL_FORMAT_PVRTC4				= 181,
				0, 										//		PVRSRV_PIXEL_FORMAT_PVRTCII2			= 182,
				0, 										//		PVRSRV_PIXEL_FORMAT_PVRTCII4			= 183,
				0, 										//		PVRSRV_PIXEL_FORMAT_PVRTCIII			= 184,
				0, 										//		PVRSRV_PIXEL_FORMAT_PVRO8				= 185,
				0, 										//		PVRSRV_PIXEL_FORMAT_PVRO88				= 186,
				0, 										//		PVRSRV_PIXEL_FORMAT_PT1					= 187,
				0, 										//		PVRSRV_PIXEL_FORMAT_PT2					= 188,
				0, 										//		PVRSRV_PIXEL_FORMAT_PT4					= 189,
				0, 										//		PVRSRV_PIXEL_FORMAT_PT8					= 190,
				0, 										//		PVRSRV_PIXEL_FORMAT_PTW					= 191,
				0, 										//		PVRSRV_PIXEL_FORMAT_PTB					= 192,
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB8,			//		PVRSRV_PIXEL_FORMAT_MONO8				= 193,
				0, 										//		PVRSRV_PIXEL_FORMAT_MONO16				= 194,
				0, 										//		PVRSRV_PIXEL_FORMAT_C0_YUYV				= 195,
				0, 										//		PVRSRV_PIXEL_FORMAT_C0_UYVY				= 196,
				0, 										//		PVRSRV_PIXEL_FORMAT_C0_YVYU				= 197,
				0, 										//		PVRSRV_PIXEL_FORMAT_C0_VYUY				= 198,
				0, 										//		PVRSRV_PIXEL_FORMAT_C1_YUYV				= 199,
				0, 										//		PVRSRV_PIXEL_FORMAT_C1_UYVY				= 200,
				0, 										//		PVRSRV_PIXEL_FORMAT_C1_YVYU				= 201,
				0, 										//		PVRSRV_PIXEL_FORMAT_C1_VYUY				= 202,
				0, 										//		PVRSRV_PIXEL_FORMAT_C0_YUV420_2P_UV		= 203,
				0, 										//		PVRSRV_PIXEL_FORMAT_C0_YUV420_2P_VU		= 204,
				0, 										//		PVRSRV_PIXEL_FORMAT_C0_YUV420_3P		= 205,
				0, 										//		PVRSRV_PIXEL_FORMAT_C1_YUV420_2P_UV		= 206,
				0, 										//		PVRSRV_PIXEL_FORMAT_C1_YUV420_2P_VU		= 207,
				0, 										//		PVRSRV_PIXEL_FORMAT_C1_YUV420_3P		= 208,
				0, 										//		PVRSRV_PIXEL_FORMAT_A2B10G10R10F		= 209,
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB888,		//		PVRSRV_PIXEL_FORMAT_B8G8R8_SINT			= 210,
				0, 										//		PVRSRV_PIXEL_FORMAT_PVRF32SIGNMASK		= 211,
				PVRSRV_PDUMP_PIXEL_FORMAT_ARGB4444,		//		PVRSRV_PIXEL_FORMAT_ABGR4444			= 212,	
				PVRSRV_PDUMP_PIXEL_FORMAT_ARGB1555,		//		PVRSRV_PIXEL_FORMAT_ABGR1555			= 213,
				PVRSRV_PDUMP_PIXEL_FORMAT_RGB565,		//		PVRSRV_PIXEL_FORMAT_BGR565				= 214,			
};
#endif //#if defined(INCLUDE_PVRSRV_PDUMP_FORMATS) && defined (PDUMP)
